Stan Math Library: stan/math/prim/scal/prob/student_t_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

student_t_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_STUDENT_T_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_STUDENT_T_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/square.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/lbeta.hpp>
15 #include <stan/math/prim/scal/fun/lgamma.hpp>
16 #include <stan/math/prim/scal/fun/digamma.hpp>
17 #include <stan/math/prim/scal/meta/length.hpp>
18 #include <stan/math/prim/scal/fun/grad_reg_inc_beta.hpp>
19 #include <stan/math/prim/scal/fun/inc_beta.hpp>
20 #include <stan/math/prim/scal/meta/include_summand.hpp>
21 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
22 #include <boost/random/student_t_distribution.hpp>
23 #include <boost/random/variate_generator.hpp>
24 #include <limits>
25 #include <cmath>
26
27 namespace stan {
28
29 namespace math {
30
31 template <typename T_y, typename T_dof, typename T_loc, typename T_scale>
32 typename return_type<T_y, T_dof, T_loc, T_scale>::type
33 student_t_cdf_log(const T_y& y, const T_dof& nu, const T_loc& mu,
34 const T_scale& sigma) {
35 typedef typename
36 stan::partials_return_type<T_y, T_dof, T_loc, T_scale>::type
37 T_partials_return;
38
39 // Size checks
40 if (!(stan::length(y) && stan::length(nu) && stan::length(mu)
41 && stan::length(sigma)))
42 return 0.0;
43
44 static const char* function("stan::math::student_t_cdf_log");
45
46 using stan::math::check_positive_finite;
47 using stan::math::check_finite;
48 using stan::math::check_not_nan;
49 using stan::math::check_consistent_sizes;
50 using stan::math::value_of;
51 using std::exp;
52
53 T_partials_return P(0.0);
54
55 check_not_nan(function, "Random variable", y);
56 check_positive_finite(function, "Degrees of freedom parameter", nu);
57 check_finite(function, "Location parameter", mu);
58 check_positive_finite(function, "Scale parameter", sigma);
59
60 // Wrap arguments in vectors
61 VectorView<const T_y> y_vec(y);
62 VectorView<const T_dof> nu_vec(nu);
63 VectorView<const T_loc> mu_vec(mu);
64 VectorView<const T_scale> sigma_vec(sigma);
65 size_t N = max_size(y, nu, mu, sigma);
66
67 OperandsAndPartials<T_y, T_dof, T_loc, T_scale>
68 operands_and_partials(y, nu, mu, sigma);
69
70 // Explicit return for extreme values
71 // The gradients are technically ill-defined, but treated as zero
72 for (size_t i = 0; i < stan::length(y); i++) {
73 if (value_of(y_vec[i]) == -std::numeric_limits<double>::infinity())
74 return operands_and_partials.value(stan::math::negative_infinity());
75 }
76
77 using stan::math::digamma;
78 using stan::math::lbeta;
79 using stan::math::inc_beta;
80 using std::pow;
81 using std::exp;
82 using std::log;
83
84 // Cache a few expensive function calls if nu is a parameter
85 T_partials_return digammaHalf = 0;
86
87 VectorBuilder<!is_constant_struct<T_dof>::value,
88 T_partials_return, T_dof>
89 digamma_vec(stan::length(nu));
90 VectorBuilder<!is_constant_struct<T_dof>::value,
91 T_partials_return, T_dof>
92 digammaNu_vec(stan::length(nu));
93 VectorBuilder<!is_constant_struct<T_dof>::value,
94 T_partials_return, T_dof>
95 digammaNuPlusHalf_vec(stan::length(nu));
96
97 if (!is_constant_struct<T_dof>::value) {
98 digammaHalf = digamma(0.5);
99
100 for (size_t i = 0; i < stan::length(nu); i++) {
101 const T_partials_return nu_dbl = value_of(nu_vec[i]);
102
103 digammaNu_vec[i] = digamma(0.5 * nu_dbl);
104 digammaNuPlusHalf_vec[i] = digamma(0.5 + 0.5 * nu_dbl);
105 }
106 }
107
108 // Compute vectorized cdf_log and gradient
109 for (size_t n = 0; n < N; n++) {
110 // Explicit results for extreme values
111 // The gradients are technically ill-defined, but treated as zero
112 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity()) {
113 continue;
114 }
115
116 const T_partials_return sigma_inv = 1.0 / value_of(sigma_vec[n]);
117 const T_partials_return t = (value_of(y_vec[n]) - value_of(mu_vec[n]))
118 * sigma_inv;
119 const T_partials_return nu_dbl = value_of(nu_vec[n]);
120 const T_partials_return q = nu_dbl / (t * t);
121 const T_partials_return r = 1.0 / (1.0 + q);
122 const T_partials_return J = 2 * r * r * q / t;
123 const T_partials_return betaNuHalf = exp(lbeta(0.5, 0.5 * nu_dbl));
124 T_partials_return zJacobian = t > 0 ? - 0.5 : 0.5;
125
126 if (q < 2) {
127 T_partials_return z
128 = inc_beta(0.5 * nu_dbl, (T_partials_return)0.5, 1.0 - r);
129 const T_partials_return Pn = t > 0 ? 1.0 - 0.5 * z : 0.5 * z;
130 const T_partials_return d_ibeta = pow(r, -0.5)
131 * pow(1.0 - r, 0.5*nu_dbl - 1) / betaNuHalf;
132
133 P += log(Pn);
134
135 if (!is_constant_struct<T_y>::value)
136 operands_and_partials.d_x1[n]
137 += - zJacobian * d_ibeta * J * sigma_inv / Pn;
138
139 if (!is_constant_struct<T_dof>::value) {
140 T_partials_return g1 = 0;
141 T_partials_return g2 = 0;
142
143 stan::math::grad_reg_inc_beta(g1, g2, 0.5 * nu_dbl,
144 (T_partials_return)0.5, 1.0 - r,
145 digammaNu_vec[n], digammaHalf,
146 digammaNuPlusHalf_vec[n],
147 betaNuHalf);
148
149 operands_and_partials.d_x2[n]
150 += zJacobian * (d_ibeta * (r / t) * (r / t) + 0.5 * g1) / Pn;
151 }
152
153 if (!is_constant_struct<T_loc>::value)
154 operands_and_partials.d_x3[n]
155 += zJacobian * d_ibeta * J * sigma_inv / Pn;
156 if (!is_constant_struct<T_scale>::value)
157 operands_and_partials.d_x4[n]
158 += zJacobian * d_ibeta * J * sigma_inv * t / Pn;
159
160 } else {
161 T_partials_return z = 1.0 - inc_beta((T_partials_return)0.5,
162 0.5*nu_dbl, r);
163 zJacobian *= -1;
164
165 const T_partials_return Pn = t > 0 ? 1.0 - 0.5 * z : 0.5 * z;
166
167 T_partials_return d_ibeta = pow(1.0-r, 0.5*nu_dbl-1) * pow(r, -0.5)
168 / betaNuHalf;
169
170 P += log(Pn);
171
172 if (!is_constant_struct<T_y>::value)
173 operands_and_partials.d_x1[n]
174 += zJacobian * d_ibeta * J * sigma_inv / Pn;
175
176 if (!is_constant_struct<T_dof>::value) {
177 T_partials_return g1 = 0;
178 T_partials_return g2 = 0;
179
180 stan::math::grad_reg_inc_beta(g1, g2, (T_partials_return)0.5,
181 0.5 * nu_dbl, r,
182 digammaHalf, digammaNu_vec[n],
183 digammaNuPlusHalf_vec[n],
184 betaNuHalf);
185
186 operands_and_partials.d_x2[n]
187 += zJacobian * (- d_ibeta * (r / t) * (r / t) + 0.5 * g2) / Pn;
188 }
189
190 if (!is_constant_struct<T_loc>::value)
191 operands_and_partials.d_x3[n]
192 += - zJacobian * d_ibeta * J * sigma_inv / Pn;
193 if (!is_constant_struct<T_scale>::value)
194 operands_and_partials.d_x4[n]
195 += - zJacobian * d_ibeta * J * sigma_inv * t / Pn;
196 }
197 }
198
199 return operands_and_partials.value(P);
200 }
201 }
202 }
203 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::lbeta
fvar< T > lbeta(const fvar< T > &x1, const fvar< T > &x2)
Definition: lbeta.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::student_t_cdf_log
return_type< T_y, T_dof, T_loc, T_scale >::type student_t_cdf_log(const T_y &y, const T_dof &nu, const T_loc &mu, const T_scale &sigma)
Definition: student_t_cdf_log.hpp:33
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::inc_beta
fvar< T > inc_beta(const fvar< T > &a, const fvar< T > &b, const fvar< T > &x)
Definition: inc_beta.hpp:20
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
grad_reg_inc_beta.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
lbeta.hpp
inc_beta.hpp
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
stan::math::grad_reg_inc_beta
void grad_reg_inc_beta(T &g1, T &g2, T a, T b, T z, T digammaA, T digammaB, T digammaSum, T betaAB)
Definition: grad_reg_inc_beta.hpp:14
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.